//I had to remove all the comments from this file because I needed more lines of code
export default class AudioManager
{
	constructor(runtime, contextOpts)
	{
		this.runtime = runtime;
		this.audioContext = null;
		
		if (typeof AudioContext !== "undefined")
			this.audioContext = new AudioContext(contextOpts);
		else if (typeof webkitAudioContext !== "undefined")
			this.audioContext = new webkitAudioContext(contextOpts);
		else
			throw new Error("AudioContext not supported");
	}
	
	decodeAudioData(arrayBuffer, isWebM)
	{
		if (this.runtime.assets.isWebMOpusSupported || !isWebM)
		{
			return new Promise((resolve, reject) =>
			{
				this.audioContext.decodeAudioData(arrayBuffer, resolve, reject);
			});
		}
		else
		{
			return this.runtime.assets.decodeWebMOpus(this.audioContext, arrayBuffer);
		}
	}
	
	async loadSound(url)
	{
		const isWebM = url.toLowerCase().endsWith(".webm");
		
		const audioUrl = this.runtime.assets.mediaFolder + url;

		const arrayBuffer = await this.runtime.assets.fetchArrayBuffer(audioUrl);

		const audioBuffer = await this.decodeAudioData(arrayBuffer, isWebM);

		return audioBuffer;
	}

	playSound(audioBuffer)
	{
		const source = this.audioContext.createBufferSource();
		source.buffer = audioBuffer;
		source.connect(this.audioContext.destination);
		source.start(0);
	}
	
	async loadMusic(url)
	{
		const isWebM = url.toLowerCase().endsWith(".webm");
		
		if (isWebM && !this.runtime.assets.isWebMOpusSupported)
			return this.loadSound(url);
		
		const audioElem = new Audio();
		
		const musicUrl = await this.runtime.assets.getMediaFileUrl(url);
		audioElem.src = musicUrl;
		
		audioElem.load();
		await this.waitForCanPlayThrough(audioElem);
		return audioElem;
	}
	
	waitForCanPlayThrough(audioElem)
	{
		return new Promise((resolve, reject) =>
		{
			audioElem.addEventListener("canplaythrough", resolve);
			audioElem.addEventListener("error", reject);
		});
	}
	
	playMusic(audio)
	{
		if (audio instanceof AudioBuffer)
		{
			this.playSound(audio);
		}
		else
		{
			audio.play();
		}
	}
}
