const enemySize = 64;
export default class EnemyMovement
{
	constructor(runtime)
	{
		this.runtime = runtime;
		this.allEnemies = null;
	}
	
	move(player)
	{
		this.allEnemies = [];
		let enemies = [];
		let flyingEnemies = [];
		
		//All enemy types, unless they need special movement
		const enemies1 = this.runtime.objects.Enemy1.getAllInstances();
		const enemies2 = this.runtime.objects.Enemy2.getAllInstances();
		const enemies3 = this.runtime.objects.Enemy3.getAllInstances();
		
		enemies = enemies.concat(enemies1, enemies2);
		flyingEnemies = flyingEnemies.concat(enemies3);
		this.allEnemies = this.allEnemies.concat(enemies, flyingEnemies);
		
		for (let i = 0; i < enemies.length; i++)
		{	
			//Physics
			const physics = enemies[i].behaviors.Physics;
			
			//Universal
			if (this.universal(enemies[i], physics)) continue;
			
			//Max velocity
			if (physics.getVelocityX() > 301) physics.setVelocity(300, 0);
			if (physics.getVelocityX() < -301) physics.setVelocity(-300, 0);
			
			//Movement
			physics.applyForceTowardPosition(40, player.x, enemies[i].y);
		}
		
		for (let i = 0; i < flyingEnemies.length; i++)
		{	
			//Physics
			const physics = flyingEnemies[i].behaviors.Physics;
			
			//Universal
			if (this.universal(flyingEnemies[i], physics)) continue;
			
			//Max velocity
			if (physics.getVelocityX() > 301) physics.setVelocity(300, 0);
			if (physics.getVelocityX() < -301) physics.setVelocity(-300, 0);
			
			physics.applyImpulse(0, -0.62);
			physics.applyForceTowardPosition(40, player.x, flyingEnemies[i].y - 1);
		}
		console.log(this.runtime.globalVars.totalEnemies);
		this.runtime.globalVars.totalEnemies = this.allEnemies.length;
	}
	
	//Returns true if the enemy should be skipped for now
	universal(enemy, physics)
	{
		//Death
		if (enemy.instVars["health"] <= 0)
		{
			this.kill(enemy);
			return true;
		}
		
		//Bullet stun
		if (enemy.instVars["bulletStun"] > 0)
		{
			enemy.instVars["bulletStun"] -= this.runtime.dt;
			return true;
		}
		
		//Direction
		if (physics.getVelocityX() > 0) enemy.width = enemySize;
		else enemy.width = -enemySize;
		
		physics.isPreventRotation = true;
	}
	
	kill(enemy)
	{
		const dust = this.runtime.objects.EnemyDeath.createInstance(0, enemy.x, enemy.y, "");
		dust.setSize(Math.abs(enemy.width), Math.abs(enemy.width));
		dust.isPreventRotation = true;
		enemy.destroy();
	}
	
	killAll()
	{
		for (let i = 0; i < this.allEnemies.length; i++) this.kill(this.allEnemies[i]);
	}
}